# Imports
import os,re
from typing import TypedDict
from .Common import change_path_form

class SrtToVtt():
    """Convert .srt file to .vtt file

    Args:
        inputfilename (path): the output file name based on the inputfile name, as well as the path.
    """

    def __init__(self, inputfilename):

        self.inputfilename = self._getInputPath(inputfilename)
        self.outputfilename = self._getOutputPath()

    def __call__(self):
        self.main()

    def _getInputPath(self, inputfilename):

        if inputfilename == None:
            print("Please enter the name of the file to open:", end=" ")
            inputfilename = input()

        done = False
        while done == False:
            if os.path.isfile(inputfilename):
                done = True
            else:
                print("File not found, please enter the filename:", end=" ")
                inputfilename = input()

        return inputfilename

    def _getOutputPath(self):
        """ Return the outputfilename depend on the inputfilename """

        path, name = os.path.split(self.inputfilename)
        # Get the file name with suffix
        name = os.path.splitext(name)[0]
        outputfilename = os.path.join(path, name+".vtt")
        return outputfilename

    def _getTransformedLine(self, line):
        if "-->" in line:
            return line.replace(",", ".")
        else:
            return line

    def main(self):

        # Open the file
        inputfile = open(self.inputfilename, "rb").read().decode("utf-8-sig").splitlines()

        output = open(self.outputfilename, "w")
        output.write("WEBVTT")
        output.write("\n\n")
        for line in inputfile:
            output.write(self._getTransformedLine(line))
            output.write("\n")
        # Close our output file
        output.close()

        print(f"Write Done : {self.outputfilename}")


class SubFuncs():
    """
    Normal Subtitll functions
    : getSubFolder()

    """

    def __init__(self, path: str):
        self.path = change_path_form(path)

    def getSubFolder(self):

        """Get the folders video name and sub name

        Returns:
            [dict]: {'fileName': '', 'subName': ''}
        """

        Type_folderData = TypedDict("Type_folderData", fileName=str, subName=str)

        videoSuffix = ['mp4', 'mkv', 'mov', 'avi']
        subSuffix = ['srt', 'ass']
        folderData: Type_folderData = dict()

        fileList: [str] = os.listdir(self.path)
        for file in fileList:
            for suffix in videoSuffix:
                if re.findall(f".{suffix}$",file):
                    folderData["fileName"] = file
                    break
            for suffix in subSuffix:
                if re.findall(f".{suffix}$", file):
                    folderData["subName"] = file
                    break

        return folderData


if __name__ == "__main__":
    test = SubFuncs("E:\Videos\Friends & 老友记\S10\es1")




from .Common import requests_retry
from pprint import pprint
from PIL import Image


def checkOneImage(path):
    ''' 判断path是否为有效图片，是返回true，否返回flase且删除文件 '''
    try:
        Image.open(path)
        return True
    except IOError:
        # os.remove(path)
        return False


def downloadImage(url, name, path, _header=None):
    '''
    下载一张图片
    
    TODO: Change the suffix 
    '''

    # print(f"Donwloading Image: {name}")

    response = requests_retry(url, _header)

    if response:

        image_path = f'{path}{name}.jpg'

        with open(image_path, 'wb') as f:
            f.write(response.content)

        if checkOneImage(image_path):
            return True
        else:
            print(f"Donloaded wrong image: {name}")
            return False

    else:
        # print(f"Failed Donload: {name}")
        return False



import time
import json
import requests
import re
import httpx

class Translation():
    def __init__(self):
        """
        Translate Function
        """
        import sys
        from importlib import reload
        reload(sys)

        self.YOUDAO_URL = 'https://openapi.youdao.com/api'
        self.APP_KEY = '55780d0d419052ba'
        self.APP_SECRET = 'cm9nQh6b4rrBfDBBEpWOD1i2eDTGkdCx'

    def encrypt(self, signStr):
        import hashlib
        hash_algorithm = hashlib.sha256()
        hash_algorithm.update(signStr.encode('utf-8'))
        return hash_algorithm.hexdigest()

    def truncate(self, q):
        if q is None:
            return None
        size = len(q)
        return q if size <= 20 else q[0:10] + str(size) + q[size - 10:size]

    def do_request(self, data):
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        return requests.post(self.YOUDAO_URL, data=data, headers=headers)

    async def do_request_async(self, data):
        headers = {'Content-Type': 'application/x-www-form-urlencoded'}
        async with httpx.AsyncClient(headers=headers) as client:
            result = await client.post(self.YOUDAO_URL, data=data)
        return result

    def trans(self, from_language, content, useby='p'):
        """
        from_language : ja = japan; en = english;
        if useby = 'p' => filter bracket content
        """
        import uuid

        if useby == 'p':
            pattern = '\(.*?\)|（.*?）|\[.*?\]'
            content = re.sub(pattern, '', content, 0)

        if content.endswith('\\') or content.endswith("'"):
            content = content[:-1]

        data = {}
        data['from'] = from_language
        data['to'] = 'zh-CHS'
        data['signType'] = 'v3'
        curtime = str(int(time.time()))
        data['curtime'] = curtime
        salt = str(uuid.uuid1())
        signStr = self.APP_KEY + self.truncate(content) + salt + curtime + self.APP_SECRET
        sign = self.encrypt(signStr)
        data['appKey'] = self.APP_KEY
        data['q'] = content
        data['salt'] = salt
        data['sign'] = sign
        #data['vocabId'] = "您的用户词表ID"

        response = self.do_request(data)
        #contentType = response.headers['Content-Type']
        result = json.loads(response.content.decode())

        if result['errorCode'] == '0':
            return result['translation'][0]
        else:
            errorcode = result['errorCode']
            print(f'Translation wrong Error Code : {errorcode} \n Retring...')

            response = self.do_request(data)
            result = json.loads(response.content.decode())

            if result['errorCode'] == '0':
                return result['translation'][0]
            else:
                errorcode = result['errorCode']
                return f'Translation wrong Error Code : {errorcode}'

    async def trans_async(self, from_language, content, useby='p'):
        """
        from_language : ja = japan; en = english;
        if useby = 'p' => filter bracket content
        """
        import uuid
        if useby == 'p':
            pattern = '\(.*?\)|（.*?）|\[.*?\]'
            content = re.sub(pattern, '', content, 0)

        if content.endswith('\\') or content.endswith("'"):
            content = content[:-1]

        data = {}
        data['from'] = from_language
        data['to'] = 'zh-CHS'
        data['signType'] = 'v3'
        curtime = str(int(time.time()))
        data['curtime'] = curtime
        salt = str(uuid.uuid1())
        signStr = self.APP_KEY + self.truncate(content) + salt + curtime + self.APP_SECRET
        sign = self.encrypt(signStr)
        data['appKey'] = self.APP_KEY
        data['q'] = content
        data['salt'] = salt
        data['sign'] = sign
        #data['vocabId'] = "您的用户词表ID"

        response = await self.do_request_async(data)
        #contentType = response.headers['Content-Type']
        result = json.loads(response.content.decode())

        if result['errorCode'] == '0':
            return result['translation'][0]
        else:
            errorcode = result['errorCode']
            print(f'Translation wrong Error Code : {errorcode} \n Retring...')

            response = await self.do_request_async(data)
            result = json.loads(response.content.decode())

            if result['errorCode'] == '0':
                return result['translation'][0]
            else:
                errorcode = result['errorCode']
                return f'Translation wrong Error Code : {errorcode}'



import smtplib
from pathlib import Path
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication
from email.mime.image import MIMEImage
from email.utils import formataddr
from email.header import Header


class Email():
    """
        :my_email = ownEmail.Email(user,password,name,host)
        :my_email.make_msg(content="test", title=f'继续测试')
        :my_email.send_msg()
    """

    def __init__(self, user='285242442@qq.com', password='tpgdlmjmfqxybgdf', name="Neely", host='smtp.qq.com'):

        self._user = user
        self._password = password
        self._host = host
        self._msg = MIMEMultipart()  # 构建主体
        self._msg['From'] = formataddr([name, user])  # 发件人

    def make_normal_msg(self, content, title, receiver='285242442@qq.com'):

        self._msg = MIMEText(
            content,
            'plain',
        )

        self._receiver = receiver
        self._msg['From'] = formataddr([self._name, self._user])
        self._msg['To'] = formataddr(['Your name', self._receiver])
        self._msg['Subject'] = title

    def make_mult_mag(self, content="", title="", file_path="", image_path="", receiver_name="", receiver='285242442@qq.com'):

        self.receiver = receiver
        self._msg['To'] = formataddr([receiver_name, receiver])  # 收件人

        if file_path:
            _path = Path(file_path)
            self._msg['Subject'] = Header(title if title else _path.stem, 'utf-8')  # 邮件主题

            with open(_path, 'rb') as file:
                attchment = MIMEApplication(file.read())

            attchment.add_header('Content-Disposition', 'attachment', filename=_path.name)
            self._msg.attach(attchment)  # 添加附件到邮件

        if image_path:
            with open(image_path, 'rb') as file:
                msgimage = MIMEImage(file.read())  # 打开图片

            html_img = f'<p>{content}<br><img src="cid:image1"></br></p>'
            msgimage.add_header('Content-ID', '<image1>')  # 设置图片
            self._msg.attach(msgimage)
            self._msg.attach(MIMEText(html_img, 'html', 'utf-8'))  # 添加到邮件正文

        else:
            self._msg['Subject'] = Header(title, 'utf-8')  # 邮件主题
            self._msg.attach(MIMEText(content, 'html', 'utf-8'))  # 添加到邮件正文

    def send_msg(self):

        try:
            smtp = smtplib.SMTP_SSL(self._host, 465)
            _ = smtp.login(self._user, self._password)
            _ = smtp.sendmail(self._user, [self.receiver], self._msg.as_string())
            _ = smtp.quit()
            return True
        except Exception as e:
            print(e)
            return False



import asyncio
import httpx
from typing import List
from tqdm import tqdm
from urllib.parse import urlparse
from pathlib import Path


def getDonwloadsPath():
    import winreg
    """ 获取windows下downloads文件夹路径 """
    key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, r'Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders')
    return winreg.QueryValueEx(key, "{374DE290-123F-4565-9164-39C4925E467B}")[0]


class AsyncDownloader(object):

    def __init__(self, thread_num: int, url: str, folder: str, filename: str | None = None, proxies=False):
        """
            Async Download

            Args:
                thread_num (int): async num
                url (str): download url
                folder (str): save path
                filename (str, optional): If Defaults to None it will get name from the url
                proxies (bool, optional): Chose if to use the 127.0.0.1:1080 sockets proxies. Defaults to False.
        """

        self.proxies = "http://127.0.0.1:1080" if proxies else None

        if not filename:
            filename = urlparse(url).path.split('/')[-1]

        self.url = url
        self.client = httpx.AsyncClient(proxies=self.proxies)
        self.thread_num = thread_num

        self.folder_path = Path(folder)  # .joinpath(Path(filename).stem)
        self.file_path = self.folder_path.joinpath(filename)

        self.file_size = self._get_file_size()
        self.cut_info = self._cutting()

        # Progress Bar
        self.tqdm_obj = tqdm(total=self.file_size, unit_scale=True, unit_divisor=1024, unit="B")

        self._create_folder(self.folder_path)

    def __call__(self):
        self.main()

    def _create_folder(self, folder_path):
        """
            Create a folder path

            Args:
                folder_path (path): normal path or Path obj
        """
        p = Path(folder_path)

        if not p.exists():
            p.mkdir()

        return

    def _get_file_size(self):

        with httpx.stream("GET", self.url, proxies=self.proxies) as res:

            size = res.headers.get("Content-Length")

            if not size:
                size = len(res.read())

            return int(size)

    def _cutting(self):
        """
        切割成若干份
        :param file_size: 下载文件大小
        :param thread_num: 线程数量
        :return:
        :[0, 31409080],
        :[31409081, 62818160],
        :[62818161, 94227240],
        :[94227241, 125636320],
        :[125636321, 157045400],
        :[157045401, 188454480],
        :[188454481, 219863560],
        :[219863561, 251272640],
        :[251272641, 282681720],
        :[282681721, '-']]
        """
        cut_info: List[List[int | str]] = []
        cut_size = self.file_size // self.thread_num

        for num in range(self.thread_num):
            cut_info.append([cut_size*num + 1, cut_size * (num + 1)])

            if num == self.thread_num - 1:
                cut_info[-1][1] = "-"
            elif num == 0:
                cut_info[0][0] = 0

        return cut_info

    def _merge_files(self):
        """
        合并分段下载的文件
        :param file_path:
        :return:
        """

        with open(self.file_path.absolute(), 'ab') as f_count:
            for index in range(self.thread_num):

                sub_file = self.folder_path.joinpath(f"{index}_{self.file_path.name}")

                with open(sub_file.absolute(), 'rb') as sub_write:
                    f_count.write(sub_write.read())

                # 合并完成删除子文件
                sub_file.unlink()

        return

    async def downloader(self, index, start_size, stop_size, retry=False):

        sub_file = self.folder_path.joinpath(f"{index}_{self.file_path.name}")

        if sub_file.exists():
            temp_size = sub_file.stat().st_size  # 本地已经下载的文件大小
            if not retry:
                self.tqdm_obj.update(temp_size)  # 更新下载进度条
        else:
            temp_size = 0

        stop_size = "" if stop_size == '-' else stop_size

        headers = {'Range': f'bytes={start_size + temp_size}-{stop_size}'}

        down_file = open(sub_file.absolute(), 'ab')

        try:
            async with self.client.stream("GET", self.url, headers=headers) as response:
                num_bytes_downloaded = response.num_bytes_downloaded
                async for chunk in response.aiter_bytes():
                    if chunk:
                        down_file.write(chunk)
                        self.tqdm_obj.update(response.num_bytes_downloaded - num_bytes_downloaded)
                        num_bytes_downloaded = response.num_bytes_downloaded

        except Exception as e:
            print("{}:请求超时,尝试重连\n报错信息:{}".format(index, e))
            await self.downloader(index, start_size, stop_size, retry=True)

        finally:
            down_file.close()

        return

    async def main_download(self):

        index = 0
        tasks = []
        for info in self.cut_info:
            task = asyncio.create_task(self.downloader(index, info[0], info[1]))
            tasks.append(task)
            index += 1

        await asyncio.gather(*tasks)
        await self.client.aclose()

    async def async_main(self):
        """ 异步中下载防止event loop """
        if self.file_path.exists():
            if self.file_path.stat().st_size >= self.file_size:
                print(f"{self.file_path.name} Already exists.")
                return

        await self.main_download()
        self._merge_files()

    def main(self):

        if self.file_path.exists():
            if self.file_path.stat().st_size >= self.file_size:
                print(f"{self.file_path.name} Already exists.")
                return

        asyncio.run(self.main_download())
        self._merge_files()


if __name__ == "__main__":

    folder = "D:/Downloads"
    url = r"https://d3.qinkan.net/d/mobi/26/%E3%80%8A%E5%87%A1%E4%BA%BA%E4%BF%AE%E4%BB%99%E4%BC%A0%E3%80%8B(%E7%B2%BE%E6%A0%A1%E7%89%88)_qinkan.net.mobi"
    AsyncDownloader(20, url, folder, proxies=False)()
 



import pymysql


class Mysql:
    '''
    数据库类
     :参数 db=选择数据库，默认为Week_event，data_dict返回数据是否需要dict形式，默认为false
     :每一种方法只需sql语句参数即可使用
     :insert,updata,delete方法没区别，便于使用时review
     :empty为清空表并排序自动递增
     :暂未写入关闭连接，因为不知道原理以及相关性能影响，待学习
    '''

    def __init__(self, host="152.136.113.50", db='personal', port=3306, user='neely', passwd='Neely2018',  data_dict=False):

        self.my_con = pymysql.connect(
            host=host, port=port, user=user, passwd=passwd, db=db, connect_timeout=1000)

        if data_dict:
            self.my_cousor = self.my_con.cursor(
                cursor=pymysql.cursors.DictCursor)
        else:
            self.my_cousor = self.my_con.cursor()

    def get_one(self, sql):
        ''' 获取一条数据 '''

        self.my_cousor.execute(sql)

        data = self.my_cousor.fetchone()

        # self.my_cousor.close()
        # self.my_con.close()

        return data

    def get_all(self, sql, singleColum=False):
        ''' 获取多条数据
            formed="single"
        '''
        self.my_cousor.execute(sql)

        data = self.my_cousor.fetchall()

        if singleColum:
            return [i[0] for i in data]

        return data

    def insert(self, sql):
        ''' 插入数据 '''
        msg = self.my_cousor.execute(sql)

        self.my_con.commit()

        return msg

    def insert_many(self, sql, param):
        ''' 插入多条数据 '''
        msg = self.my_cousor.executemany(sql, param)

        self.my_con.commit()

        return msg

    def update(self, sql):
        ''' 更新数据 '''
        msg = self.my_cousor.execute(sql)

        self.my_con.commit()

        return msg

    def delete(self, sql):
        ''' 删除数据 '''
        msg = self.my_cousor.execute(sql)

        self.my_con.commit()

        return msg

    def empty(self, db):
        '''
        直接清空表并排序key

        用于重新写入全部数据
        '''

        sql = (f'DELETE from {db}')
        sql2 = (f'alter table {db} AUTO_INCREMENT 1')

        msg = self.my_cousor.execute(sql)
        self.my_cousor.execute(sql2)
        self.my_con.commit()

        print(f'已删除{msg}条数据，清空{db}表')



from typing import TypedDict
import httpx


class CrackCode(object):
    """
        Identify verification code

        postPic(image:bytes|url,codetype):

        postPicAsync(image:bytes|url,codetype):

        codetype : http://www.chaojiying.com/price.html

        return:

        err_no=int, err_str=str, pic_id=str, pic_str=str, md5=str
    """

    ResultModel = TypedDict("ResultModel", err_no=int, err_str=str, pic_id=str, pic_str=str, md5=str)

    def __init__(self, headers: dict | None = None):
        """
        'pass2': md5(password.encode('utf8')).hexdigest(),
        """
        self.base_data = {
            'user': 'liangxiuyu',
            'pass2': "b9c23cc6007dcec86a7577770790e534",
            'softid': '927673',  # 用户中心>>软件ID 生成一个替换 96001
            'len_min': 4
        }
        self.headers = headers or {
            'Connection': 'Keep-Alive',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.71 Safari/537.36 Edg/97.0.1072.55',
        }

        self.base_url = "http://upload.chaojiying.net/Upload/Processing.php"

    def postPic(self, image: bytes | str, codetype: str = "1004") -> ResultModel:
        """
        image: 图片字节
        codetype: 题目类型 参考 http://www.chaojiying.com/price.html
        """
        print("-> Nomal identifing verification code...")
        # Isinstance image bytes or Url link
        if isinstance(image, bytes):
            files = {'userfile': ('code.jpg', image)}
        else:
            res = httpx.get(image, headers=self.headers)
            files = {'userfile': ('code.jpg', res.content)}

        self.base_data.update({'codetype': codetype})
        response = httpx.post(self.base_url, data=self.base_data, files=files, headers=self.headers)
        return response.json()

    async def postPicAsync(self, image: bytes | str, codetype: str = "1004") -> ResultModel:

        print("-> Async identifing verification code...")

        client = httpx.AsyncClient(headers=self.headers)

        # Isinstance image bytes or Url link
        if isinstance(image, bytes):
            files = {'userfile': ('code.jpg', image)}
        else:
            res = await client.get(image)
            files = {'userfile': ('code.jpg', res.content)}

        self.base_data.update({'codetype': codetype})

        response = await client.post(self.base_url, data=self.base_data, files=files)

        _ = await client.aclose()

        # print("Code Result:",response.text)
        return response.json()

    def reportError(self, image_id):
        """
        im_id:报错题目的图片ID
        """
        print("-> Report to server this error")
        self.base_data.update({
            'id': image_id,
        })
        r = httpx.post('http://upload.chaojiying.net/Upload/ReportError.php', data=self.base_data, headers=self.headers)
        return r.json()



from .Common import *
from .Download import *
from .ExtractFile import ExtractFile
from .Image import *
from .Mysql import Mysql
from .Time import *
from .Translation import Translation
from .UncompressRAR import uncompress
from .Subtitles import SrtToVtt, SubFuncs
from .CrackCode import CrackCode
from .ownEmail import Email
from ._SSH import SSHConnection
from .Loger import Logger



import os
import shutil

class ExtractFile:
    """ 
    Input Path & Pattern to extract files the match the pattern from the path's folders into this path 
    
    If pattern is none, extrcat all kind of files. 
    """

    def __init__(self, path, pattern):
        self.path = self._changePathForm(path)
        self.pattern = pattern

    def __call__(self, *args, **kwds):
        self.main()

    def _changePathForm(self, path: str):
        new_path = eval(repr(path).replace('\\', '/'))
        return new_path

    def _getCurrentFiles(self):
        return os.listdir(self.path)

    def _getFolders(self):
        currentFiles = self._getCurrentFiles()

        folderList = list()
        for file in currentFiles:
            if os.path.isdir(self._changePathForm(os.path.join(self.path, file))):
                print(file)
                folderList.append(file)

        return folderList

    def _extractFile(self, folder):
        forlderPath = self._changePathForm(os.path.join(self.path, folder))
        folderFiles = os.listdir(forlderPath)

        for file in folderFiles:
            if file.find(self.pattern) != -1:
                shutil.move(self._changePathForm(os.path.join(forlderPath, file)), self.path)
                print(f"Move: {file}")

    def main(self):
        forlders = self._getFolders()
        for forlder in forlders:
            self._extractFile(forlder)



import time
import re
import datetime
from time import gmtime, strftime


def find_date(content):

    pattern = "\\d{4}-\\d{1,2}-\\d{1,2}"
    result = re.findall(pattern, content)
    if result:
        return result[0]
    else:
        return ""


def get_t_time():

    return str(time.time())[:10]


def get_d_time():
    return datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')


def get_short_d_time():

    return datetime.datetime.now().strftime('%Y-%m-%d')


def translate_timestamp(timestamp: int, form: str = '%Y-%m-%d %H:%M:%S'):
    """ 将时间戳转换为form格式化字符串.默认：%Y-%m-%d %H:%M:%S """
    formatted_time = datetime.datetime.fromtimestamp(timestamp).strftime(form)
    return formatted_time


def convert_second(second):

    res = strftime("%H:%M:%S", gmtime(second))
    return (res)


def translate_time(_time, data_type):
    '''
    时间转换格式

    _time = 输入的时间

    data_type = 想要的格式：
        t_time = 时间戳形式
        d_time = datetime格式
    '''

    if data_type == 't_time':
        return int(time.mktime(time.strptime(_time, "%Y-%m-%d %H:%M:%S")))
    if data_type == 'd_time':
        timeArray = time.localtime(_time)
        return time.strftime("%Y-%m-%d %H:%M:%S", timeArray)


def convert_datetime(date: str):

    date_list = re.findall(r'([0-9]{4})\D([0-3]?[0-9])\D([0-3]?[0-9])', date)

    if date_list:
        date_list = [int(i) for i in date_list[0]]
        datetime_date = datetime.datetime(date_list[0], date_list[1], date_list[2])
        return datetime_date

    else:
        raise BaseException("Not validated date string.")


def calculate_date(date: str, num: int, format: str = "%Y-%m-%d"):
    """
        Calculate date with given args

        Args:
            date (str): %Y-%m-%d
            num (int): +num or -num
            format(str) : default is %Y-%m-%d

        Returns:
            str: format
    """

    datetime_date = convert_datetime(date)
    result = (datetime_date + datetime.timedelta(days=num)).strftime(format)

    return result


def trans_to_date(value: str | int | datetime.date| datetime.datetime, format: str = "%Y-%m-%d"):
    """ 判断类型并转换为date对象 """

    # 判断是否为数字字符串
    if isinstance(value, str) and value.isdigit():
        value = int(value)

    if isinstance(value, datetime.date):
        return value

    elif isinstance(value, datetime.datetime):
        return value.date()

    elif isinstance(value, str):
        try:
            return datetime.datetime.strptime(value, format).date()
        except ValueError:
            raise ValueError(f"输入的字符串格式不正确,请使用<{format}>格式")
    elif isinstance(value, int):
        try:
            return datetime.datetime.fromtimestamp(value).date()
        except ValueError:
            raise ValueError("输入的时间戳无效")
    else:
        raise TypeError("输入值的类型必须是字符串(str)或整数(int)")


def trans_to_datetime(value: str | int):
    _date = trans_to_date(value)
    return datetime.datetime.combine(_date, datetime.time())



import paramiko
from typing import cast


class SSHConnection:
    # 初始化连接创建Transport通道
    def __init__(self, host='127.0.0.1', port=22, user='root', pwd='root'):
        self.host = host
        self.port = port
        self.user = user
        self.pwd = pwd
        self.__transport = paramiko.Transport((self.host, self.port))
        self.__transport.connect(username=self.user, password=self.pwd)
        self.sftp = cast(paramiko.SFTPClient, paramiko.SFTPClient.from_transport(self.__transport))

    # 关闭通道
    def close(self):
        self.sftp.close()
        self.__transport.close()

    # 上传文件到远程主机
    def upload(self, local_path, remote_path):
        _ = self.sftp.put(local_path, remote_path)
        return _

    # 从远程主机下载文件到本地
    def download(self, local_path, remote_path):
        self.sftp.get(remote_path, local_path)

    # 在远程主机上创建目录
    def mkdir(self, target_path, mode=777):
        self.sftp.mkdir(target_path, mode)

    # 删除远程主机上的目录
    def rmdir(self, target_path):
        self.sftp.rmdir(target_path)

    # 查看目录下文件以及子目录（如果需要更加细粒度的文件信息建议使用listdir_attr）
    def listdir(self, target_path):
        return self.sftp.listdir(target_path)

    # 删除文件
    def remove(self, target_path):
        self.sftp.remove(target_path)

    # 查看目录下文件以及子目录的详细信息（包含内容和参考os.stat返回一个FSTPAttributes对象，对象的具体属性请用__dict__查看）
    def listdirattr(self, target_path):
        list = []
        try:
            list = self.sftp.listdir_attr(target_path)
        except BaseException as e:
            print(e)

        return list

    # 获取文件详情
    def stat(self, remote_path):
        return self.sftp.stat(remote_path)

    # SSHClient输入命令远程操作主机
    def cmd(self, command):
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy)
        ssh._transport = self.__transport  # type:ignore
        stdin, stdout, stderr = ssh.exec_command(command)
        result = stdout.read()
        print(result)
        return result



import datetime
import json
import requests
import re
from retry import retry
from pprint import pprint
import functools
import time


class DateEncode(json.JSONEncoder):
    '''
    TypeError: Object of type 'datetime' is not JSON serializable 解决方法：


    这个错误的原因是json.dumps无法对字典中的datetime时间格式数据进行转化，dumps的原功能是将dict转化为str格式，不支持转化时间，所以需要将json类部分内容重新改写，来处理这种特殊日期格式。

    使用方式： json.dumps(dict,cls=DateEncoder)

    '''
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime("%Y-%m-%d %H:%M:%S")
        elif isinstance(obj, datetime.date):
            return obj.strftime("%Y-%m-%d")
        elif isinstance(obj, datetime.timedelta):
            return obj.seconds
        else:
            return json.JSONEncoder.default(self, obj)

def creat_cookies(from_path: str, to_path: str):
    with open(from_path, "r") as file:
        cookies = json.loads(file.read())

    real_cookies = {}
    for item in cookies['cookies']:
        real_cookies[item['name']] = item['value']

    with open(to_path, "w") as file:
        _ = file.write(json.dumps(real_cookies, indent=4))

def get_run_time(info="used"):
    def _time_me(fn):
        print("Begin to timing:", fn.__name__)

        @functools.wraps(fn)
        def _wrapper(*args, **kwargs):
            start = time.perf_counter()
            print("Start at:", start)
            fn(*args, **kwargs)
            print("%s %s %s" % (fn.__name__, info, time.perf_counter() - start), "second")

        return _wrapper

    return _time_me


def get_headers(copys):
    headers = copys
    headers = headers.strip().split('\n')
    # 使用字典生成式将参数切片重组，并去掉空格，处理带协议头中的://
    headers = {x.split(':')[0].strip(): ("".join(x.split(':')[1:])).strip().replace('//', "://") for x in headers}
    # 使用json模块将字典转化成json格式打印出来

    return headers


def get_suffix(filename):
    """ Get file's suffix """
    return re.findall(r'\.[^.\\/:*?"<>|\r\n]+$', filename)[0]


def search_torrent(content) -> str | None:

    # In case name like : hhd1080
    content = re.sub("hhd", "", content)
    content = re.sub("chd", "", content)
    content = re.sub("1080", "", content)
    torrent_pattern = '[a-z]{2,5}[0-9]{3,5}|[A-Z]{2,5}-[0-9]{3,5}'

    torrent = re.search(torrent_pattern, content)
    torrent = torrent.group() if torrent else None

    if torrent:
        part_pattern = '([a-zA-Z]{2,5})-([0-9]{2,5})'
        name_part = re.search(part_pattern, torrent)

        if name_part:

            first_part = name_part.group(1).lower()
            second_part = name_part.group(2)

            for _ in range(5 - len(second_part)):
                second_part = '0' + second_part

            torrent = first_part + second_part

    return torrent


def isChinese(word):
    """ 判断word中是否含有汉字 """
    for ch in word:
        if '\u4e00' <= ch <= '\u9fff':
            return True
    return False


def isJapanese(word):
    """ 判断word中是否含有日语 """
    for ja in word:
        if '\u0800' <= ja <= '\u4e00':
            return True
    return False


def change_path_form(path: str):
    new_path = eval(repr(path).replace('\\', '/'))
    return new_path


@retry(tries=3, delay=3)
def requests_retry(url, _header=None):
    ''' 提取并分析一页内容时的requests方法，使用retry装饰器自动重试，失败则抛出异常)

    成功返回response 错误返回false'''
    if _header:
        response = requests.get(url, headers=_header, timeout=10)
    else:
        response = requests.get(url, timeout=10)

    if response.status_code == 200:
        return response
    else:
        # print(f'Response status code wrong : {response.status_code}')
        return False


def write_json(all_data, file_name='test', ensure=True):

    with open(f'{file_name}.json', 'w', encoding='utf-8') as f:
        f.write(json.dumps(all_data, indent=4, ensure_ascii=ensure))


if __name__ == '__main__':
    print(time.perf_counter())



from loguru import logger
from pathlib import Path


class Logger():
    # 日志简单配置

    def __init__(self, logPath: str | Path, need_log=True,serialize=False):
        self.my_logger = logger

        # 判断是否需要写入日志
        if need_log is True:

            log_path = Path(logPath)

            if not log_path.parent.exists():
                Path.mkdir(log_path.parent, parents=True, exist_ok=True)

            _ = self.my_logger.add(log_path, retention="30 days", encoding='utf-8', enqueue=True,serialize=serialize)

    def info(self, content):

        self.my_logger.info(content)

    def debug(self, content):
        self.my_logger.debug(content)

    def error(self, content):
        self.my_logger.error(content)

    def critical(self, content):
        self.my_logger.critical(content)

    def warning(self, content):
        self.my_logger.warning(content)

    def success(self, content):
        self.my_logger.success(content)

    def trace(self, content):
        self.my_logger.trace(content)

    def traceback(self,content=""):
        self.my_logger.exception(content)



import rarfile

def uncompress(src_file, dest_dir):
    """解压各种类型的压缩包

    :param src_file: 你要解压的压缩包文件
    :type src_file: file
    :param dest_dir: 你要解压到的目标路径
    :type dest_dir: str
    """

    # 需要安装rar包：pip install rarfile
    rar = rarfile.RarFile(src_file)
    rar.extractall(dest_dir)
    rar.close()



from setuptools import setup, find_packages

setup(
    name='xiuyutools',
    version='1.1',
    packages=find_packages(),
    description='Tools for myself',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='Neely',
    author_email='liangxiuyu@outlook.com',
    url='https://github.com/liangxiuyu/owntools',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
    ],
)
